;*******************************************************************************
;*
;*      Titel:  Unterprogramme zur Nutzung der 1-Wire-Schnittstelle
;*              (SR_1_Wire.asm)
;*
;*      Datum:  26/02/11        Version:        0.2
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A        Quarzfrequenz:  16MHz
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*
;*      1 Pull-Up-Widerstand von 4,7k
;*      Daten werden mit r24 uebergeben. Der Status nach dem Ausfuehren
;*      einer Funktion befindet sich in r25
;*      
;*	Folgende Tabellen im SRAM müssen reserviert sein:
;*	ROMCOD:  .BYTE 8  
;*	SCRPAD1: .BYTE 72 	;Sensorzahl*9 Byte grosses Scratchpad-Tabelle                                
;*	TMPASC1: .BYTE 56       ;ttt,tt+Space  7 Byte*8 Sensoren
;*	SCRPAD2: .BYTE 72 	;nur falls 2 Linie!		
;*	TMPASC2: .BYTE 56 	;nur falls 2 Linie!	
;*	
;*	Die Pinnummer wird auch im Hauptprogramm festgelegt:
;*      .EQU    OW1PNr = 0
;*      .EQU    OW2PNr = 1	;nur falls 2 Linie!	
;*      
;*      
;*      Copyright (c) 2011   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************
.EQU    LineCnt = 1	;bei 2 Linien: LineCnt = 2

;===============================================================================
;       1 Linie
;===============================================================================
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU    OW1DDR    = DDRC	;Port definieren (Pin Nummer im Hauptprogramm!)
.EQU    OW1Port   = PORTC
.EQU    OW1Pin    = PINC
.EQU    OW1PNr    = 2
;-------------------------------------------------------------------------------
;       Initialisiere den 1-Wire_Bus (Master: Reset Pulse, Slave Presence Pulse)
;-------------------------------------------------------------------------------
OW1INI: sbi     OW1DDR,OW1PNr           ;Ausgang
        cbi     OW1Port,OW1PNr          ;Erzeuge Reset Pulse (min. 480us)
        rcall   W500us
        sbi     OW1Port,OW1PNr
        cbi     OW1DDR,OW1PNr           ;Eingang
        rcall   W50us                   ;Warte auf Presence Pulse (61-74us)
        rcall   W10us
        rcall   W10us
        sbic    OW1Pin,OW1PNr
        rjmp    OW1INI                  ;Falls kein Presence Pulse, dann nochmal
        rcall   W100us
        ret

;-------------------------------------------------------------------------------
;       1-Wire_Bus: 1 Byte schreiben (Daten in r24)
;-------------------------------------------------------------------------------        
OW1SND: push    r17
        sbi     OW1DDR,OW1PNr           ;Ausgang
        ldi     r17,8
OW1SN1: ror     r24
        brcs    OW1SN2
        cbi     OW1Port,OW1PNr          ;1-Wire_Bus: 0 schreiben
        rcall   W100us
        sbi     OW1Port,OW1PNr
        rcall   W10us
        rjmp    OW1SN3
OW1SN2: cbi     OW1Port,OW1PNr          ;1-Wire_Bus: 1 schreiben
        rcall   W10us
        sbi     OW1Port,OW1PNr
        rcall   W100us
OW1SN3: dec     r17
        brne    OW1SN1
        pop     r17
        ret

;-------------------------------------------------------------------------------
;       1-Wire_Bus: 1 Byte lesen (Daten in r24, Bit in T-Flag)
;-------------------------------------------------------------------------------
OW1RCV: push    r17
        ldi     r17,8
OW1RC1: sbi     OW1DDR,OW1PNr             ;Bit lesen (Bit in T-Flag), Ausgang
        cbi     OW1Port,OW1PNr            ;Master aktiviert Leitung fuer 2us
        rcall   W1us
        rcall   W1us
        sbi     OW1Port,OW1PNr
        cbi     OW1DDR,OW1PNr             ;Eingang
        rcall   W10us
        sbic    OW1Pin,OW1PNr
        rjmp    OW1RC2
        clt
        rjmp    OW1RC3
OW1RC2: set
OW1RC3: rcall   W100us
        brts    OW1RC4
        clc
        ror     r24
        rjmp    OW1RC5
OW1RC4: sec
        ror     r24
OW1RC5: dec     r17
        brne    OW1RC1
        pop     r17
        ret

;-------------------------------------------------------------------------------
;       Starten und Match ROM senden
;-------------------------------------------------------------------------------
OW1MROM:
        rcall   OW1INI                  ;Reset Pulse senden, Presence P. warten
        ldi     r24,0x55                ;Befehl "Match ROM" senden
        rcall   OW1SND
        lds     r24,ROMCOD
        rcall   OW1SND
        lds     r24,ROMCOD+1
        rcall   OW1SND
        lds     r24,ROMCOD+2
        rcall   OW1SND
        lds     r24,ROMCOD+3
        rcall   OW1SND
        lds     r24,ROMCOD+4
        rcall   OW1SND
        lds     r24,ROMCOD+5
        rcall   OW1SND
        lds     r24,ROMCOD+6
        rcall   OW1SND
        lds     r24,ROMCOD+7
        rcall   OW1SND
        ret

;-------------------------------------------------------------------------------
;       9 Byte Scratchpad lesen und im SRAM (SCRPAD) abspeichern
;       Offset des Scrpad in r24 (Achtung beginnt mit 0!)
;       Scratchpad-Tabelle muss für (Sensoranzahl)*9 Bytes reserviert sein!
;-------------------------------------------------------------------------------
OW1RSCR:
        push    r16
        push    r18
        push    XL
        push    XH

        ldi     r16,9
        mul     r24,r16
        ldi     XL,LOW(SCRPAD1)
        ldi     XH,HIGH(SCRPAD1)
        add     XL,r0
        adc     XH,r1
        ldi     r24,0xBE                ;Befehl "Read Scratchpad" senden
        rcall   OW1SND
        ldi     r18,9                   ;Scratchpad schreiben
OW1RSC1:
        rcall   OW1RCV
        st      X+,r24
        dec     r18
        brne    OW1RSC1

        pop     XH
        pop     XL
        pop     r18
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Temperatur ermitteln, wenn nur 1 Sensor vorhanden ist
;       Die SN muss schon in der ROMCODE-Tabelle stehen
;-------------------------------------------------------------------------------
OW11DS:	rcall   OW1MROM                 ;Match ROM Code senden
        ldi     r24,0x44                ;Befehl "Convert T" senden
        rcall   OW1SND
	rcall   W1s		        ;Warten bis Wandlung fertig
	
	rcall   OW1MROM                 ;Match ROM senden
        clr	r24                     ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   OW1RSCR                 ;Lese Scratchpad uns speichere im SRAM
        clr	r24                     ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   S1TTOA                  ;Berechne Temp. und wandele in ASCII
        clr	r24                     ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall	TMP1_232		;zur EIA232
        ret

;-------------------------------------------------------------------------------
;       Convert T senden am mehrere DS18x20
;       Anzahl der Sensoren in r24
;       ROM Codes in Flash-Tabelle SNxTAB (siehe unten)
;-------------------------------------------------------------------------------
OW1SCON:
        push    r16
        push    r17
        push    r18
        push    XL
        push    XH
        push    ZL
        push    ZH

        mov     r25,r24                 ;r25 dient als Sensorzaehler (r24-0)
        clr     r17                     ;r17 dient als Sensorzaehler (0-r24)
OW1SCO1:
        ldi     r16,8
        mul     r17,r16
        ldi     ZL,LOW(SN1TAB*2)        ;Romcode in den SRAM laden
        ldi     ZH,HIGH(SN1TAB*2)
        add     ZL,r0
        adc     ZH,r1
        ldi     XL,LOW(ROMCOD)
        ldi     XH,HIGH(ROMCOD)
        ldi     r18,8
OW1SCO2:
        lpm     r16,Z+
        st      X+,r16
        dec     r18
        brne    OW1SCO2
        rcall   OW1MROM                 ;Match ROM Code senden
        ldi     r24,0x44                ;Befehl "Convert T" senden
        rcall   OW1SND
        inc     r17
        dec     r25                     ;naechster Sensor
        brne    OW1SCO1

        pop     ZH
        pop     ZL
        pop     XH
        pop     XL
        pop     r18
        pop     r17
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Temperatur mehrerer Sensoren lesen
;       Anzahl der Sensoren in r24
;       ROM Codes in Flash-Tabelle SNxTAB (siehe unten)
;       Alle Daten in Scratchpad-Tabelle
;       ASCII-Daten in TMPASCx-Tabelle
;-------------------------------------------------------------------------------
OW1RTMP:
        push    r16
        push    r17
        push    r18
        push    XL
        push    XH
        push    ZL
        push    ZH

        mov     r25,r24                 ;r25 dient als Sensorzaehler (r24-0)
        clr     r17                     ;r17 dient als Sensorzaehler (0-r24)
OW1RTM1:
        ldi     r16,8
        mul     r17,r16
        ldi     ZL,LOW(SN1TAB*2)        ;Romcode in den SRAM laden
        ldi     ZH,HIGH(SN1TAB*2)
        add     ZL,r0
        adc     ZH,r1
        ldi     XL,LOW(ROMCOD)
        ldi     XH,HIGH(ROMCOD)
        ldi     r18,8
OW1RTM2:
        lpm     r16,Z+
        st      X+,r16
        dec     r18
        brne    OW1RTM2
        rcall   OW1MROM                 ;Match ROM senden
        mov     r24,r17                 ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   OW1RSCR                 ;Lese Scratchpad uns speichere im SRAM
        mov     r24,r17                 ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   S1TTOA                  ;Berechne Temp.r und wandele in ASCII
        inc     r17
        dec     r25                     ;naechster Sensor
        brne    OW1RTM1

        pop     ZH
        pop     ZL
        pop     XH
        pop     XL
        pop     r18
        pop     r17
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Berechnung der Temperatur und Umwandlung in ASCII (DS18S20)
;       Parameteruebergabe: Temperatur aus Scratchpad, Offset Scrpad in r24
;       (Achtung beginnt mit 0!)
;       Resultat SRAM-Tabelle TMPASCx
;       (8 Byte, ttt,tt+Space+NullByte)* Sensoranzahl
;-------------------------------------------------------------------------------
S1TTOA: push    r16
        push    r17
        push    r25
        push    ZL
        push    ZH
        push    YL
        push    YH

        ldi     r16,9			;Zeiger initilaisieren
        mul     r24,r16
        ldi     ZL,LOW(SCRPAD1)
        ldi     ZH,HIGH(SCRPAD1)
        add     ZL,r0
        adc     ZH,r1
        ldi     r16,7
        mul     r24,r16
        ldi     YL,LOW(TMPASC1)
        ldi     YH,HIGH(TMPASC1)
        add     YL,r0
        adc     YH,r1
        
        lds	r16,ROMCOD+0
        cpi	r16,0x28
        brne	S1TTOS
        rjmp	S1TTOB


S1TTOS: ldi     r16,'0'                 ;Startwert
        std     Y+0,r16
        ldd     r16,Z+1                 ;Temperatur negativ?
        cpi     r16,0xFF
        brne    S1TTO1
        ldi     r16,'-'                 ;Minuszeichen
        std     Y+0,r16
        ldd     r16,Z+0                 ;Zweierkomplement
        com     r16
        inc     r16
        std     Z+0,r16
S1TTO1: ldd     r0,Z+0                  ;Absolutwert
        lsr     r0                      ;Kommastelle weg = TEMP_READ
        ldi     r16,100                 ;mit 100 multiplizieren (2 Kommastellen)
        mul     r0,r16                  ;Resultat in r1:r0
        mov     r24,r0
        mov     r25,r1
        sbiw    r24,25                  ;0,25 abziehen
        ldd     r16,Z+6                 ;COUNT_REMAIN aus Scratchpad
        ldi     r17,16
        sub     r17,r16
        ldi     r16,100
        mul     r17,r16
        ldi     r17,16
        mov     r2,r17
        rcall   DIV21
        mov     r16,r1
        cpi     r16,8                    ;Runde (8/16)
        brlo    S1TTO2
        inc     r0
S1TTO2: add     r24,r0
        adc     r25,Zero        
        ldi     r17,0x30                ;nach ASCII
        mov     r0,r24
        mov     r1,r25
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21                   ;Resultat/100
        mov     r24,r1                  ;Rette Rest nach r24
        clr     r1
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21                   ;/100  Trennen von Zehner und Hunderter
        ldd     r16,Y+0
        cpi     r16,'-'
        breq    S1TTO3
        add     r0,r17
        std     Y+0,r0
S1TTO3: mov     r0,r1
        clr     r1
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21                   ;/10 zum Trennen von Einer und Zehner
        add     r0,r17
        std     Y+1,r0
        add     r1,r17
        std     Y+2,r1
        ldi     r16,','                 ;Komma
        std     Y+3,r16
        mov     r0,r24
        clr     r1
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21                   ;/10 zum Trennen von Einer und Zehner
        add     r0,r17
        std     Y+4,r0
        add     r1,r17
        std     Y+5,r1
        ldi     r16,' '			;Leerzeichen
        std     Y+6,r16
        clr     r16			;Nullbyte
        std     Y+7,r16

	rjmp	S1TTOR



	;DS18B20! (andere Berechnung!) 
S1TTOB: ldi     r16,'0'                 ;Startwert
        std     Y+0,r16
        ldd     r16,Z+1                 ;Temperatur negativ?
        andi	r16,0xF8
        cpi     r16,0xF8
        brne    S1TTO4
        ldi     r16,'-'                 ;Minuszeichen
        std     Y+0,r16
        ldd     r16,Z+0                 ;Zweierkomplement
        com     r16
        inc     r16
        std     Z+0,r16
        
        
S1TTO4: ldd     r16,Z+1                 ;Temperatur groesser 99,9 dann eine 1
	cpi	r16,0x06		;an die dritte Stelle
	breq	S1TTO5
	brge	S1TTO6
S1TTO5: cpi	r16,0x40
	brge	S1TTO6
	rjmp	S1TTO7
S1TTO6: ldi     r17,'1'                 
        std     Y+0,r17
S1TTO7: swap	r16			;Zahl
	ldd     r17,Z+0
	andi	r17,0xF0
	swap	r17
	add	r16,r17
	clr	r1
	mov	r0,r16
	ldi	r16,100			;/10 zum Abtrennen von Hunderter
	mov	r2,r16
	rcall   DIV21
	mov	r0,r1
	clr	r1
	ldi	r16,10			;/10 zum Trennen von Einer und Zehner
	mov	r2,r16
	rcall   DIV21
	ldi     r17,0x30                ;nach ASCII
	add	r0,r17
	std     Y+1,r0
	add	r1,r17
	std     Y+2,r1
	ldi     r16,','                 ;Komma
        std     Y+3,r16
	ldd     r16,Z+0			;Kommastellen
	andi	r16,0x0F
	ldi	r17,5			;x625 = 5x125
	mul	r16,r17
	mov	r16,r0
	ldi	r17,125			;x625 = 5x125
	mul	r16,r17                 ;Resultat in r1:r0
	ldi	r16,100			;nur 2 Stellen!
	mov	r2,r16
	rcall   DIV21
	mov	r16,r0
	cpi	r16,50			;Runden
	brlo	S1TTO8	
	inc	r0			
S1TTO8: clr	r1
	ldi	r16,10			;/10 zum Trennen von Zehntel
	mov	r2,r16			;und Hunderstel
	rcall   DIV21
	ldi     r17,0x30                ;nach ASCII
	add	r0,r17
	std     Y+4,r0
	add	r1,r17
	std     Y+5,r1
        ldi     r16,' '			;Leerzeichen
        std     Y+6,r16
        clr     r16			;Nullbyte
        std     Y+7,r16

S1TTOR: pop     YH
        pop     YL
        pop     ZH
        pop     ZL
        pop     r25
        pop     r17
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       ROM Code ermitteln (Tabelle ROMCOD im SRAM)
;-------------------------------------------------------------------------------
OW1RROM:
        rcall   OW1INI          ;Reset Pulse senden, auf Presence Pulse warten
        ldi     r24,0x33        ;Befehl "Read ROM" senden
        rcall   OW1SND
        rcall   OW1RCV          ;8 Byte ROM Code lesen und im SRAM abspeichern
        sts     ROMCOD,r24
        rcall   OW1RCV
        sts     ROMCOD+1,r24
        rcall   OW1RCV
        sts     ROMCOD+2,r24
        rcall   OW1RCV
        sts     ROMCOD+3,r24
        rcall   OW1RCV
        sts     ROMCOD+4,r24
        rcall   OW1RCV
        sts     ROMCOD+5,r24
        rcall   OW1RCV
        sts     ROMCOD+6,r24
        rcall   OW1RCV
        sts     ROMCOD+7,r24
        ret

;-------------------------------------------------------------------------------
;       Sende Seriennummer (ROM Code) zur seriellen Schnittstelle
;-------------------------------------------------------------------------------
ROM232: ldi	r24,'"'
        rcall	SNDCHR
	lds     r24,ROMCOD
	rcall	HTOA
        lds     r24,ROMCOD+1
        rcall	HTOA
        lds     r24,ROMCOD+2
        rcall	HTOA
        lds     r24,ROMCOD+3
        rcall	HTOA
        lds     r24,ROMCOD+4
        rcall	HTOA
        lds     r24,ROMCOD+5
        rcall	HTOA
        lds     r24,ROMCOD+6
        rcall	HTOA
        lds     r24,ROMCOD+7
        rcall	HTOA
        ldi	r24,'"'
        rcall	SNDCHR
        ret

HTOA:	push	r16	
	push	r17
	ldi	r16,0x30
	ldi	r17,0x37
	
	mov	r25,r24
	andi	r24,0xF0
	swap	r24	
	cpi	r24,10
	brge	HTOA1
	add	r24,r16	
	rjmp	HTOA2
HTOA1:	add	r24,r17	
HTOA2:	rcall	SNDCHR	
	mov	r24,r25
	andi	r24,0x0F
	cpi	r24,10
	brge	HTOA3
	add	r24,r16	
	rjmp	HTOA4
HTOA3:	add	r24,r17		
HTOA4:	rcall	SNDCHR	
	ldi	r24,' '
	rcall	SNDCHR
	
	pop	r17
	pop	r16
	ret


;-------------------------------------------------------------------------------
;       Sende Scratchpad1 zur seriellen Schnittstelle
;       Offset in r24 (beginnt mit 0!)
;-------------------------------------------------------------------------------
SCR1_232:
        push    r16
        push    r18
        push    XL
        push    XH

        ldi     r16,9
        mul     r24,r16
        ldi     XL,LOW(SCRPAD1)
        ldi     XH,HIGH(SCRPAD1)
        add     XL,r0
        adc     XH,r1
        ldi     r18,9           ;Scratchpad schreiben
SCR1_L: ld      r24,X+
        rcall   SNDCHR
        dec     r18
        brne    SCR1_L

        pop     XH
        pop     XL
        pop     r18
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Sende Temperatur zur seriellen Schnittstelle
;       Offset in r24 (beginnt mit 0!)
;-------------------------------------------------------------------------------
TMP1_232:
        push    r16
        push    r18
        push    r24
        push    XL
        push    XH

        ldi     r16,7
        mul     r24,r16
        ldi     XL,LOW(TMPASC1)
        ldi     XH,HIGH(TMPASC1)
        add     XL,r0
        adc     XH,r1
        ldi     r18,7           ;Scratchpad schreiben
TMP1_L: ld      r24,X+
        rcall   SNDCHR
        dec     r18
        brne    TMP1_L

pop     XH
        pop     XL
        pop     r24
        pop     r18
        pop     r16
        ret


.IF LineCnt == 2

;===============================================================================
;       2 Linie
;===============================================================================
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU    OW2DDR    = DDRB
.EQU    OW2Port   = PORTB
.EQU    OW2Pin    = PINB

;-------------------------------------------------------------------------------
;       Initialisiere den 1-Wire_Bus (Master: Reset Pulse, Slave Presence Pulse)
;-------------------------------------------------------------------------------
OW2INI: sbi     OW2DDR,OW2PNr           ;Ausgang
        cbi     OW2Port,OW2PNr          ;Erzeuge Reset Pulse (min. 480us)
        rcall   W500us
        sbi     OW2Port,OW2PNr
        cbi     OW2DDR,OW2PNr           ;Eingang
        rcall   W50us                   ;Warte auf Presence Pulse (61-74us)
        rcall   W10us
        rcall   W10us
        sbic    OW2Pin,OW2PNr
        rjmp    OW2INI                  ;Falls kein Presence Pulse, dann nochmal
        rcall   W100us
        ret

;-------------------------------------------------------------------------------
;       1-Wire_Bus: 1 Byte schreiben (Daten in r24)
;-------------------------------------------------------------------------------
OW2SND: push    r17
        sbi     OW2DDR,OW2PNr           ;Ausgang
        ldi     r17,8
OW2SN1: ror     r24
        brcs    OW2SN2
        cbi     OW2Port,OW2PNr          ;1-Wire_Bus: 0 schreiben
        rcall   W100us
        sbi     OW2Port,OW2PNr
        rcall   W10us
        rjmp    OW2SN3
OW2SN2: cbi     OW2Port,OW2PNr          ;1-Wire_Bus: 1 schreiben
        rcall   W10us
        sbi     OW2Port,OW2PNr
        rcall   W100us
OW2SN3: dec     r17
        brne    OW2SN1
        pop     r17
        ret
        
;-------------------------------------------------------------------------------
;       1-Wire_Bus: 1 Byte lesen (Daten in r24, Bit in T-Flag)
;-------------------------------------------------------------------------------
OW2RCV: push    r17
        ldi     r17,8
OW2RC1: sbi     OW2DDR,OW2PNr             ;Bit lesen (Bit in T-Flag), Ausgang
        cbi     OW2Port,OW2PNr            ;Master aktiviert Leitung fuer 2us
        rcall   W1us
        rcall   W1us
        sbi     OW2Port,OW2PNr
        cbi     OW2DDR,OW2PNr             ;Eingang
        rcall   W10us
        sbic    OW2Pin,OW2PNr
        rjmp    OW2RC2
        clt
        rjmp    OW2RC3
OW2RC2: set
OW2RC3: rcall   W100us
        brts    OW2RC4
        clc
        ror     r24
        rjmp    OW2RC5
OW2RC4: sec
        ror     r24
OW2RC5: dec     r17
        brne    OW2RC1
        pop     r17
        ret

;-------------------------------------------------------------------------------
;       Starten und Match ROM senden
;-------------------------------------------------------------------------------
OW2MROM:
        rcall   OW2INI                  ;Reset Pulse senden, Presence P. warten
        ldi     r24,0x55                ;Befehl "Match ROM" senden
        rcall   OW2SND
        lds     r24,ROMCOD
        rcall   OW2SND
        lds     r24,ROMCOD+1
        rcall   OW2SND
        lds     r24,ROMCOD+2
        rcall   OW2SND
        lds     r24,ROMCOD+3
        rcall   OW2SND
        lds     r24,ROMCOD+4
        rcall   OW2SND
        lds     r24,ROMCOD+5
        rcall   OW2SND
        lds     r24,ROMCOD+6
        rcall   OW2SND
        lds     r24,ROMCOD+7
        rcall   OW2SND
        ret

;-------------------------------------------------------------------------------
;       9 Byte Scratchpad lesen und im SRAM (SCRPADx) abspeichern
;       Offset des Scrpad in r24 (Achtung beginnt mit 0!)
;       Scratchpad-Tabelle muss für (Sensoranzahl)*9 Bytes reserviert sein!
;-------------------------------------------------------------------------------
OW2RSCR:
        push    r16
        push    r18
        push    XL
        push    XH

        ldi     r16,9
        mul     r24,r16
        ldi     XL,LOW(SCRPAD2)
        ldi     XH,HIGH(SCRPAD2)
        add     XL,r0
        adc     XH,r1
        ldi     r24,0xBE                ;Befehl "Read Scratchpad" senden
        rcall   OW2SND
        ldi     r18,9                   ;Scratchpad schreiben
OW2RSC1:
        rcall   OW2RCV
        st      X+,r24
        dec     r18
        brne    OW2RSC1

        pop     XH
        pop     XL
        pop     r18
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Convert T senden am mehrere DS18x20
;       Anzahl der Sensoren in r24
;       ROM Codes in Flash-Tabelle SNxTAB (siehe unten)
;-------------------------------------------------------------------------------
OW2SCON:
        push    r16
        push    r17
        push    r18
        push    XL
        push    XH
        push    ZL
        push    ZH

        mov     r25,r24                 ;r25 dient als Sensorzaehler (r24-0)
        clr     r17                     ;r17 dient als Sensorzaehler (0-r24)
OW2SCO1:
        ldi     r16,8
        mul     r17,r16
        ldi     ZL,LOW(SN2TAB*2)        ;Romcode in den SRAM laden
        ldi     ZH,HIGH(SN2TAB*2)
        add     ZL,r0
        adc     ZH,r1
        ldi     XL,LOW(ROMCOD)
        ldi     XH,HIGH(ROMCOD)
        ldi     r18,8
OW2SCO2:
        lpm     r16,Z+
        st      X+,r16
        dec     r18
        brne    OW2SCO2
        rcall   OW2MROM                 ;Match ROM Code senden
        ldi     r24,0x44                ;Befehl "Convert T" senden
        rcall   OW2SND
        inc     r17
        dec     r25                     ;naechster Sensor
        brne    OW2SCO1

        pop     ZH
        pop     ZL
        pop     XH
        pop     XL
        pop     r18
        pop     r17
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Temperatur mehrerer Sensoren lesen
;       Anzahl der Sensoren in r24
;       ROM Codes in Flash-Tabelle SNxTAB (siehe unten)
;       Alle Daten in Scratchpad-Tabelle
;       ASCII-Daten in TMPASCx-Tabelle
;-------------------------------------------------------------------------------
OW2RTMP:
        push    r16
        push    r17
        push    r18
        push    XL
        push    XH
        push    ZL
        push    ZH

        mov     r25,r24                 ;r25 dient als Sensorzaehler (r24-0)
        clr     r17                     ;r17 dient als Sensorzaehler (0-r24)
OW2RTM1:
        ldi     r16,8
        mul     r17,r16
        ldi     ZL,LOW(SN2TAB*2)        ;Romcode in den SRAM laden
        ldi     ZH,HIGH(SN2TAB*2)
        add     ZL,r0
        adc     ZH,r1
        ldi     XL,LOW(ROMCOD)
        ldi     XH,HIGH(ROMCOD)
        ldi     r18,8
OW2RTM2:
        lpm     r16,Z+
        st      X+,r16
        dec     r18
        brne    OW2RTM2
        rcall   OW2MROM                 ;Match ROM senden
        mov     r24,r17                 ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   OW2RSCR                 ;Lese Scratchpad uns speichere im SRAM
        mov     r24,r17                 ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   S2TTOA                  ;Berechne Temp.r und wandele in ASCII
        inc     r17
        dec     r25                     ;naechster Sensor
        brne    OW2RTM1

        pop     ZH
        pop     ZL
        pop     XH
        pop     XL
        pop     r18
        pop     r17
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Berechnung der Temperatur und Umwandlung in ASCII (DS18S20)
;       Parameteruebergabe: Temperatur aus Scratchpad, Offset Scrpad in r24
;       (Achtung beginnt mit 0!)
;       Resultat SRAM-Tabelle TMPASCx (7 Byte, ttt,tt+Space)* Sensoranzahl
;-------------------------------------------------------------------------------
S2TTOA: push    r16
        push    r17
        push    r25
        push    ZL
        push    ZH
        push    YL
        push    YH

        ldi     r16,9
        mul     r24,r16
        ldi     ZL,LOW(SCRPAD2)
        ldi     ZH,HIGH(SCRPAD2)
        add     ZL,r0
        adc     ZH,r1
        ldi     r16,7
        mul     r24,r16
        ldi     YL,LOW(TMPASC2)
        ldi     YH,HIGH(TMPASC2)
        add     YL,r0
        adc     YH,r1

        ldi     r16,'0'                 ;Startwert
        std     Y+0,r16
        ldd     r16,Z+1                 ;Temperatur negativ?
        cpi     r16,0xFF
        brne    S2TTOP
        ldi     r16,'-'                 ;Minuszeichen
        std     Y+0,r16
        ldd     r16,Z+0                 ;Zweierkomplement
        com     r16
        inc     r16
        std     Z+0,r16
S2TTOP: ldd     r0,Z+0                  ;Absolutwert
        lsr     r0                      ;Kommastelle weg = TEMP_READ
        ldi     r16,100                 ;mit 100 multiplizieren (2 Kommastellen)
        mul     r0,r16                  ;Resultat in r1:r0
        mov     r24,r0
        mov     r25,r1
        sbiw    r24,25                  ;0,25 abziehen
        ldd     r16,Z+6                 ;COUNT_REMAIN aus Scratchpad
        ldi     r17,16
        sub     r17,r16
        ldi     r16,100
        mul     r17,r16
        ldi     r17,16
        mov     r2,r17
        rcall   DIV21
        mov     r16,r1
        cpi     r16,8                    ;Runde (8/16)
        brlo    S2TTO1
        inc     r0
S2TTO1: add     r24,r0
        adc     r25,Zero
        ldi     r17,0x30                ;nach ASCII
        mov     r0,r24
        mov     r1,r25
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21                   ;Resultat/100
        mov     r24,r1                  ;Rette Rest nach r24
        clr     r1
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21                   ;/100  Trennen von Zehner und Hunderter
        ldd     r16,Y+0
        cpi     r16,'-'
        breq    S2TTO2
        add     r0,r17
        std     Y+0,r0
S2TTO2: mov     r0,r1
        clr     r1
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21                   ;/10 zum Trennen von Einer und Zehner
        add     r0,r17
        std     Y+1,r0
        add     r1,r17
        std     Y+2,r1
        ldi     r16,','                 ;Komma
        std     Y+3,r16
        mov     r0,r24
        clr     r1
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21                   ;/10 zum Trennen von Einer und Zehner
        add     r0,r17
        std     Y+4,r0
        add     r1,r17
        std     Y+5,r1
        ldi     r16,' '
        std     Y+6,r16

        pop     YH
        pop     YL
        pop     ZH
        pop     ZL
        pop     r25
        pop     r17
        pop     r16
        ret
        
;-------------------------------------------------------------------------------
;       ROM Code ermitteln
;-------------------------------------------------------------------------------
OW2RROM:
        rcall   OW2INI          ;Reset Pulse senden, auf Presence Pulse warten
        ldi     r24,0x33        ;Befehl "Read ROM" senden
        rcall   OW2SND
        rcall   OW2RCV          ;8 Byte ROM Code lesen und im SRAM abspeichern
        sts     ROMCOD,r24
        rcall   OW2RCV
        sts     ROMCOD+1,r24
        rcall   OW2RCV
        sts     ROMCOD+2,r24
        rcall   OW2RCV
        sts     ROMCOD+3,r24
        rcall   OW2RCV
        sts     ROMCOD+4,r24
        rcall   OW2RCV
        sts     ROMCOD+5,r24
        rcall   OW2RCV
        sts     ROMCOD+6,r24
        rcall   OW2RCV
        sts     ROMCOD+7,r24
        ret

;-------------------------------------------------------------------------------
;       Sende Scratchpad2 zur seriellen Schnittstelle
;       Offset in r24 (beginnt mit 0!)
;-------------------------------------------------------------------------------
SCR2_232:
        push    r16
        push    r18
        push    XL
        push    XH

        ldi     r16,9
        mul     r24,r16
        ldi     XL,LOW(SCRPAD2)
        ldi     XH,HIGH(SCRPAD2)
        add     XL,r0
        adc     XH,r1
        ldi     r18,9           ;Scratchpad schreiben
SCR2_L: ld      r24,X+
        rcall   SNDCHR
        dec     r18
        brne    SCR2_L

        pop     XH
        pop     XL
        pop     r18
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Sende Temperatur zur seriellen Schnittstelle
;       Offset in r24 (beginnt mit 0!)
;-------------------------------------------------------------------------------
TMP2_232:
        push    r16
        push    r18
        push    r24
        push    XL
        push    XH

        ldi     r16,7
        mul     r24,r16
        ldi     XL,LOW(TMPASC2)
        ldi     XH,HIGH(TMPASC2)
        add     XL,r0
        adc     XH,r1
        ldi     r18,7           ;Scratchpad schreiben
TMP2_L: ld      r24,X+
        rcall   SNDCHR
        dec     r18
        brne    TMP2_L

        pop     XH
        pop     XL
        pop     r24
        pop     r18
        pop     r16
        ret

.ENDIF

;-------------------------------------------------------------------------------
;       Ganzzahldivision 2Byte / 1Byte
;-------------------------------------------------------------------------------
;Dividend:       r1,r0 (LSB)
;Divisor         r2
;Schleifenzähler r16
;Resultat in r0 + Rest in r1
;Achtung Ueberlauf wenn obere n Bit des Dividenden groesser gleich n Bit Divisor

DIV21:  push    r16
        in      r16,SREG
        push    r16

        ldi     r16,8           ;Schleifenzaehler init. (Bitzahl Quotient)

DIV21A: lsl     r0              ;Rotiere Dividend 1 Stelle nach links
        rol     r1              ;rechts wird Null reingeschoben
        brcs    DIV21B          ;Falls Carry muss Subtraktion erfolgen!
        cp      r1,r2           ;Subtraktion testen
        brlo    DIV21C          ;Ueberspringe Subtraktion, wenn kleiner

DIV21B: sub     r1,r2           ;Subtraktion durchfuehren
        inc     r0              ;Bit 0 im Dividenden setzen

DIV21C: dec     r16             ;Schleifenzaehler dekrementieren
        brne    DIV21A

        pop     r16
        out     SREG,r16
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       Tabellen mit Serial Codes
;-------------------------------------------------------------------------------
SN1TAB: .DB     0x10, 0xE0, 0xAF, 0xED, 0x01, 0x08, 0x00, 0x87  ;1
        .DB     0x10, 0x8F, 0xD7, 0xED, 0x01, 0x08, 0x00, 0x33  ;2
        .DB     0x10, 0xF9, 0xA4, 0xED, 0x01, 0x08, 0x00, 0x3A  ;3
        .DB     0x10, 0x25, 0xF0, 0xED, 0x01, 0x08, 0x00, 0x09  ;4
        .DB     0x10, 0xEA, 0xB7, 0xED, 0x01, 0x08, 0x00, 0x0A  ;5
        .DB     0x10, 0xB5, 0xEC, 0xED, 0x01, 0x08, 0x00, 0xE5  ;6
        .DB     0x10, 0x12, 0xA3, 0xED, 0x01, 0x08, 0x00, 0xBA  ;7

SN2TAB: .DB     0x10, 0xEA, 0xB7, 0xED, 0x01, 0x08, 0x00, 0x0A  ;5
        .DB     0x10, 0xB5, 0xEC, 0xED, 0x01, 0x08, 0x00, 0xE5  ;6
        .DB     0x10, 0x12, 0xA3, 0xED, 0x01, 0x08, 0x00, 0xBA  ;7