;*******************************************************************************
;*
;*      Titel:  Unterprogramm zur Nutzung des EEPROM 24LC256 ueber I2C
;*              (SR_EEPROM_24LC256.asm)
;*
;*      Datum:  16/01/11        Version:        0.1
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A        Quarzfrequenz:  16MHz
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*
;*
;*******************************************************************************
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU    ERRLED   = 1            ;1 falls Fehler LED erwuenscht sons 0
.EQU    ERRDDR   = DDRD         ;LED zur Anzeige eines Uebetragungsfehler
.EQU    ERRPORT  = PORTD
.EQU    ERRPNr   = 7

;-------------------------------------------------------------------------------
;       Datenbyte speichern
;       Adresse im SRAM Label E24ADH und E24ADL
;       Datenbyte im SRAM Label: E24BYTE
;-------------------------------------------------------------------------------
E24SB:  rcall   I2CSTA          ;Startbit senden + Kontrolle
        cpi     r25,0x08
        brne    E24SBE
        ldi     r24,0xA0        ;Adresse des EEPROM 24LC256 senden (A0=A1=A2=0)
        rcall   I2CSND          ;(Bit 2^0 = 0 schreiben) + Kontrolle
        cpi     r25,0x18
        brne    E24SBE
        lds     r24,E24ADH      ;Adresszeiger HIGH senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24SBE
        lds     r24,E24ADL      ;Adresszeiger Low senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24SBE
        ;Datenbyte senden + Kontrolle
        lds     r24,E24BYTE      ;Datenbyte
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24SBE
        rjmp    E24SBR
E24SBE: .IF (ERRLED == 1)
        sbi     ERRDDR,ERRPNr   ;Ausgang fuer Fehler LED initialisieren
        sbi     ERRPORT,ERRPNr  ;Fehler LED ein
        .ENDIF
E24SBR: rcall   I2CSTO          ;Stoppbit senden
        rcall   E245ms          ;Es werden 5ms benötigt um eine Seite (Page) zu
                                ;beschreiben (auch wenn nur 1 Byte geschr. wird)
        ret

;-------------------------------------------------------------------------------
;	Datenseite (Page, 64 Byte) speichern
;       Adresse im SRAM Label E24ADH und E24ADL
;       64 Datenbyte in SRAM-Tabelle Label: E24PAGE
;-------------------------------------------------------------------------------
E24SP:  push    r16
        push    XL
        push    XH
        rcall   I2CSTA          ;Startbit senden + Kontrolle
        cpi     r25,0x08
        brne    E24SPE
        ldi     r24,0xA0        ;Adresse des EEPROM 24LC256 senden (A0=A1=A2=0)
        rcall   I2CSND          ;(Bit 2^0 = 0 schreiben) + Kontrolle
        cpi     r25,0x18
        brne    E24SPE
        lds     r24,E24ADH      ;Adresszeiger HIGH senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24SPE
        lds     r24,E24ADL    ;Adresszeiger Low senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24SPE
        ;64 Datenbyte senden + Kontrolle
        ldi     r16,64
        ldi     XL,LOW(E24PAGE)
        ldi     XH,HIGH(E24PAGE)
E24SPL: ld      r24,X+
        rcall   I2CSND
        cpi     r25,0x28        
        brne    E24SPE
        dec     r16
        brne    E24SPL
        rjmp    E24SPR
E24SPE: .IF (ERRLED == 1)
        sbi     ERRDDR,ERRPNr   ;Ausgang fuer Fehler LED initialisieren
        sbi     ERRPORT,ERRPNr  ;Fehler LED ein
        .ENDIF
E24SPR: rcall   I2CSTO          ;Stoppbit senden
        pop     XH
        pop     XL        
        pop     r16
        rcall   E245ms          ;Es werden 5ms benötigt um eine Seite (Page) zu
                                ;beschreiben
        ret

;-------------------------------------------------------------------------------
;	Daten empfangen (beliebige Adresse, Random Read)
;       Adresse im SRAM Label E24ADH und E24ADL
;       Datenbyte im SRAM Label: E24BYTE
;-------------------------------------------------------------------------------
E24RR:  rcall	I2CSTA		;Startbit senden + Kontrolle
        cpi     r25,0x08        
        breq    E24RR1
        cpi     r25,0x10        ;repeated start falls zu schell fuer Stoppbit
        brne    E24RRE
E24RR1: ldi     r24,0xA0        ;Adresse des EEPROM 24LC256 senden (A0=A1=A2=0)
        rcall   I2CSND          ;(Bit 2^0 = 0 schreiben) + Kontrolle
        cpi     r25,0x18
        brne    E24RRE
        lds     r24,E24ADH      ;Adresszeiger HIGH senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24RRE
        lds     r24,E24ADL      ;Adresszeiger Low senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24RRE
        rcall   I2CSTA          ;Repeated Startbit senden + Kontrolle
        cpi     r25,0x10
        brne    E24RRE
        ldi     r24,0xA1        ;Adresse des EEPROM 24LC256 senden (A0=A1=A2=0)
        rcall   I2CSND          ;(Bit 2^0 = 1 lesen) + Kontrolle
        cpi     r25,0x40
        brne    E24RRE
        ;1 Datenbyte lesen + Kontrolle
	rcall   I2CRNA          ;Datenbyte lesen Achtung! mit NAK abschliessen
        sts     E24BYTE,r24
        cpi     r25,0x58
        brne    E24RRE
        rjmp    E24RRR
E24RRE: .IF (ERRLED == 1)
        sbi     ERRDDR,ERRPNr   ;Ausgang fuer Fehler LED initialisieren
        sbi     ERRPORT,ERRPNr  ;Fehler LED ein
        .ENDIF
E24RRR: rcall   I2CSTO          ;Stoppbit senden
        ret

;-------------------------------------------------------------------------------
;       Daten empfangen (64 Byte, beliebige Adresse, Sequential Read)
;       Adresse im SRAM Label E24ADH und E24ADL
;       64 Datenbyte in der SRAM Tabelle mit Label: E24PAGE
;-------------------------------------------------------------------------------
E24RS:  push    r16
        push    XL
        push    XH
        rcall   I2CSTA          ;Startbit senden + Kontrolle
        cpi     r25,0x08
        breq    E24RS1
        cpi     r25,0x10        ;repeated start falls zu schell fuer Stoppbit
        brne    E24RSE
E24RS1: ldi     r24,0xA0        ;Adresse des EEPROM 24LC256 senden (A0=A1=A2=0)
        rcall   I2CSND          ;(Bit 2^0 = 0 schreiben) + Kontrolle
        cpi     r25,0x18
        brne    E24RSE
        lds     r24,E24ADH      ;Adresszeiger HIGH senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24RSE
        lds     r24,E24ADL      ;Adresszeiger Low senden + Kontrolle
        rcall   I2CSND
        cpi     r25,0x28
        brne    E24RSE
        rcall   I2CSTA          ;Repeated Startbit senden + Kontrolle
        cpi     r25,0x10
        brne    E24RSE
        ldi     r24,0xA1        ;Adresse des EEPROM 24LC256 senden (A0=A1=A2=0)
        rcall   I2CSND          ;(Bit 2^0 = 1 lesen) + Kontrolle
        cpi     r25,0x40
        brne    E24RSE
        ;64 Datenbyte lesen + Kontrolle
        ldi     r16,63
        ldi     XL,LOW(E24PAGE)
        ldi     XH,HIGH(E24PAGE)
E24RSL: rcall   I2CRAK          ;Datenbyte lesen mit ACK abschliessen
        st      X+,r24
        cpi     r25,0x50
        brne    E24RSE
        dec     r16
        brne    E24RSL
        rcall   I2CRNA          ;Datenbyte lesen Achtung! mit NAK abschliessen
        st      X,r24
        cpi     r25,0x58
        brne    E24RSE
        rjmp    E24RSR
E24RSE: .IF (ERRLED == 1)
        sbi     ERRDDR,ERRPNr   ;Ausgang fuer Fehler LED initialisieren
        sbi     ERRPORT,ERRPNr  ;Fehler LED ein
        .ENDIF
E24RSR: rcall   I2CSTO          ;Stoppbit senden
        pop     XH
        pop     XL
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       16-Bit Zeitschleife von 5ms (CPU_Clock in SR_I2C definiert!)
;-------------------------------------------------------------------------------        
E245ms: push    XL      ;rette verwendete Registerinhalte
        push    XH
        ;Initialisiere den Schleifenzaehler X (G = (t-tT)/4tT = f*t/4
        ldi     XL,LOW ((CPU_Clock*5)/4000)
        ldi     XH,HIGH((CPU_Clock*5)/4000)
E245mL: sbiw    XL,1    ;Dekrementiere den Schleifenzaehler 
        brne    E245mL  ;Verlasse die Schleife bei Schleifenzaehler X=0
        pop     XH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     XL
        ret             ;Ruecksprung 