;*******************************************************************************
;*
;*	Titel:	MICES2 Test 1 (mices2_test1.asm)
;*	Datum:	12/07/11	Version:	0.1	
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*	Prozessor:	ATmega32		Quarzfrequenz: 16MHz	 
;*	Eingaenge:      8 Schalter an Port B, 4 Taster an Port A (PA4-PA7)
;*	Ausgaenge:      8 Segmente an Port C, 4 Stellen an Port D (PD4-PD7)
;*			4 gelbe LEDs an Port D (PD4-PD7)
;*			
;*	TxD und RxD an PD0 und PD1 ueber Jumper		
;* 
;*      Die serielle Schnittstelle wird mit Hilfe eines Nullmodemkabel mit einem
;*      PC verbunden auf dem eine Modemsoftware laeuft (2 Jumper zum uC gesetzt) 
;*      
;*      Informationen zur Funktionsweise:
;*      
;*      Das Programm dient dazu die serielle Schnittstelle, die Schalter und
;*      Taster sowie die Sieben-Segmentanzeige des mices2-Board 
;*      (Grundbeschaltung) zu testen. Die 16 LEDs werden manuell durch Verbinden
;*      (Drahtbruecke) mit VCC getestet.
;*      Die Zustaende der vier Taster (PA3-PA7) werden an den vier gelben LEDs
;*      ausgegeben. Die 8 Schalter schalten die 8 Segmente der Sieben-Segment-
;*      Anzeige.     
;*      Echo-Betrieb des USART (Interrupt-Betrieb) 
;*      Das vom PC aus versendete Zeichen wird vom ATMEGA zurueckgesendet.
;*      8N1 / 38400 Baud (Datenformat / Übertragungsgeschwindigkeit) .      
;*			
;*******************************************************************************

;-------------------------------------------------------------------------------
;	Einbinden der controllerspezifischen Definitionsdatei
;-------------------------------------------------------------------------------
.NOLIST				;List-Output ausschalten
.INCLUDE "m32def.inc"		;AVR-Definitionsdatei einbinden
.LIST				;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
;.DSEG                          ;was ab hier folgt kommt in den SRAM-Speicher
;TAB1:  .BYTE  100              ;100 Byte grosse Tabelle im Datensegment

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG	0x0000			;Programm beginnt an der FLASH-Adresse 0x0000
RESET1:	rjmp	INIT		;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;	Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)	
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    URXCaddr                ;Vektor Nr 14 (Adresse 0x1A) fuer RXINT
        rjmp RXINT

;-------------------------------------------------------------------------------
;	Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF	Zero = r15		;Register 1 wird zum Rechnen benoetigt
	clr	r15		;und mit Null belegt
.DEF	Tmp1 = r16		;Register 16 dient als erster Zwischenspeicher
.DEF	Tmp2 = r17		;Register 17 dient als zweiter Zwischenspeicher
.DEF	Cnt1 = r18		;Register 18 dient als Zaehler
.DEF    WL = r24                ;Register 24 und 25 dienen als universelles
.DEF    WH = r25                ;Doppelregister W und zur Parameteruebergabe

	;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
	ldi	Tmp1,HIGH(RAMEND);RAMEND (SRAM) ist in der Definitions-
	out	SPH,Tmp1	 ;datei festgelegt	
	ldi	Tmp1,LOW(RAMEND)
	out	SPL,Tmp1

	;8 Pins von Port C als Ausgang initialisieren (Segmente)
        ser     Tmp1
        out     DDRC,Tmp1 
        ;4 obere Pins von Port D als Ausgang initialisieren 
        ldi     Tmp1,0xF0       
        out     DDRD,Tmp1 
        ;4 Pins von Port A als Ausgang initialisieren, andere 4 Bits als Eing. 
        ldi     Tmp1,0xF0       
        out     DDRA,Tmp1     
        ldi     Tmp1,0x0F       ;4*Pull-Up
        out     PORTA,Tmp1     
        ;8 Pin Port B als Eingang mit Pull-Up
	clr	Tmp1
	out	DDRB,Tmp1
	ser     Tmp1
	out	PORTB,Tmp1
	
        ;USART initialisieren
        ;Empfaenger einschalten, Interrupt freigeben und Datenformat festlegen
        sbi     UCSRB,RXEN      ;Epfaenger einschalten (TxEnable)
        sbi     UCSRB,TXEN      ;Sender einschalten (TxEnable)
        sbi     UCSRB,RXCIE     ;Interrupt RXCIE freigeben
        ldi     Tmp1,0x86       ;8N1
        out     UCSRC,Tmp1      
        ;Baudrate initialisieren
        clr     Tmp1            ;Teiler fuer Baudrate festlegen
        out     UBRRH,Tmp1      ;Higbyte = 0
        ldi     Tmp1,0x19       ;Teile durch dezimal 25 fuer 38400 Baud (16MHz)
        out     UBRRL,Tmp1      ;Lowbyte = 0x19

        ;Interrupt einschalten und freigeben    
        sei                     ;Interrupts global freigeben (I-Bit im SREG) 

;-------------------------------------------------------------------------------
;       Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:   ;Spiegeln der 4 Taster:
	in 	Tmp1,PINA	
	andi	Tmp1,0x0F	;Maske erstellen
	swap	Tmp1
	in 	Tmp2,PINA	;LEDs schalten
	or	Tmp2,Tmp1	
	ori	Tmp1,0x0F
	and	Tmp2,Tmp1
	ori	Tmp2,0x0F	;Pull-Ups	
	out 	PORTA,Tmp2
	
	;Spiegeln der 8 Schalter auf die Segmente
	;Multiplexing der 4 Stellen
	in 	Tmp1,PINB
	com	Tmp1	
	out 	PORTC,Tmp1
	sbi	PORTD,4
	rcall	W1ms
	cbi	PORTD,4
	sbi	PORTD,5
	rcall	W1ms
	cbi	PORTD,5
	sbi	PORTD,6
	rcall	W1ms
	cbi	PORTD,6
	sbi	PORTD,7
	rcall	W1ms
	cbi	PORTD,7
 
        rjmp    MAIN            ;Endlosschleife

;-------------------------------------------------------------------------------
;       Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
RXINT:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16             ;

	in      r16,UDR         ;Zeichen einlesen	
RXINT1: sbis	UCSRA,UDRE	;Polling ob frei
	rjmp	RXINT1
        out     UDR,r16         ;Gib ein Zeichen aus
                
        pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti                    ;zurueck und Interrupts wieder freigeben

.INCLUDE "SR_TIME_16M.asm"

;-------------------------------------------------------------------------------
.EXIT				;Ende des Quelltextes
