;*******************************************************************************
;*
;*	Titel:	MICES2 Test 3 (mices2_test3.asm)
;*	Datum:	22/07/11	Version:	0.1	
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*	Prozessor:	ATmega32		Quarzfrequenz: 16MHz	 
;*	Eingaenge:      PA0-PA2 Tastatur Spalten, 
;*	Ausgaenge:	PD4-PD7 Schrittmotor (I1-I4), PB0-PB6 LCD-Display
;*			(D4-D7,RS,E),PA3-PA6 Tastatur Zeilen, PC3 Sekunden-LED
;*	SCL an PC0
;*	SDA an PC1
;*	1-Wire an PC2
;*	TxD und RxD an PD0 und PD1 ueber Jumper
;*	SQW (Ausgang RTC) an INT0 (PD2)
;*	
;*      Die serielle Schnittstelle wird mit Hilfe eines Nullmodemkabel mit einem
;*      PC verbunden auf dem eine Modemsoftware laeuft (2 Jumper zum uC gesetzt) 
;*      
;*      Informationen zur Funktionsweise:
;*      
;*      Das Programm dient zum Testen der I2C-Schnittstelle (RTC), der 1-Wire
;*      Schnittstelle (DS1820), einer 3*4 Matrixtastatur, eines LCD-Display und
;*      eines Schrittmotors (Treiberbaustein).
;*      Zeit, Datum (RTC) und Temperatur (DS1820) werden auf dem LCD angezeigt
;*      und ueber die serielle Schnittstelle ausgegeben.
;*      8N1 / 38400 Baud (Datenformat / Übertragungsgeschwindigkeit) .      
;*      Datum und Uhrzeit koennen ueber die serielle Schnittstelle gesetzt
;*      werden. Dazu muss der Grossbuchstabe T+Leerzeichen+Datum+Zeit
;*      (ohne Zwischenzeichen und ohne Sekunden) gesendet werden.
;*      Bsp: T 2207111012 (22 Juli 2011 10h12)
;*      Das Druecken einer beliebigen Taste auf der Matrixtastatur wird
;*      auf dem LCD und ueber die serielle Schnittstelle angezeigt.
;*      Der Ausgang des RTC liefert ein Rechtecksignal mit einer Frequenz von
;*      1Hz an den Interrupt-Eingang INT0. Mit diesem Signal blinkt eine LED im
;*      Sekundentakt (PC3) und der Schrittmotor bewegt sich um einen Schritt pro
;*      Sekunde.
;*			
;*******************************************************************************

;-------------------------------------------------------------------------------
;	Einbinden der controllerspezifischen Definitionsdatei
;-------------------------------------------------------------------------------
.NOLIST				;List-Output ausschalten
.INCLUDE "m32def.inc"		;AVR-Definitionsdatei einbinden
.LIST				;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG				;was ab hier folgt kommt in den SRAM-Speicher
RTCTAB:	 .BYTE 8                ;8 Byte grosse Tabelle fuer die Echtzeituhr
RTCASC:  .BYTE 19               ;dd.mm.yy hh.MM.ss+Space+NullByte  19 Byte
ROMCOD:  .BYTE 8                ;8 Byte grosser ROM Code des DS18x20
SCRPAD1: .BYTE 72               ;Sensorzahl*9 Byte grosses Scratchpad-Tabelle
                                ;2*8 Sensoren                                
TMPASC1: .BYTE 64               ;ttt,tt+Space+Nullbyte  8 Byte*8 Sensoren

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG	0x0000			;Programm beginnt an der FLASH-Adresse 0x0000
RESET1:	rjmp	INIT		;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;	Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)	
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    INT0addr                ;Vektor fuer INT0
        rjmp ISRI0
.ORG    URXCaddr                ;Vektor fuer Interrupt RX (EIA232)
        rjmp RXINT

;-------------------------------------------------------------------------------
;	Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF	Zero = r15		;Register 1 wird zum Rechnen benoetigt
	clr	r15		;und mit Null belegt
.DEF	Tmp1 = r16		;Register 16 dient als erster Zwischenspeicher
.DEF	Tmp2 = r17		;Register 17 dient als zweiter Zwischenspeicher
.DEF	Cnt1 = r18		;Register 18 dient als Zaehler
.DEF	Cnt2 = r19		;Register 19 dient als Zaehler2 (Schrittmotor)
.DEF	KeyCode = r20		;Register 20 fuer die Tastatur
.DEF    WL = r24                ;Register 24 und 25 dienen als universelles
.DEF    WH = r25                ;Doppelregister W und zur Parameteruebergabe

	;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
	ldi	Tmp1,HIGH(RAMEND);RAMEND (SRAM) ist in der Definitions-
	out	SPH,Tmp1	 ;datei festgelegt	
	ldi	Tmp1,LOW(RAMEND)
	out	SPL,Tmp1

	;PC3 als Ausgang (Sekunden-LED)
	sbi	DDRC,3
	;PD3-7 Ausgang (Schrittmotor)
	sbi	DDRD,4
	sbi	DDRD,5
	sbi	DDRD,6
	sbi	DDRD,7
	
        clr     Cnt2		;Schrittmotorzaehler initialisieren

	;externe Interrupts initialisieren
        in      Tmp1,MCUCR              ;MCU Control Register einlesen
        andi    Tmp1,0b11111101         ;beide Flanken fuer INT0
        ori     Tmp1,0b00000001         ;
        out     MCUCR,Tmp1              ;Wert ins MCUCR zurückschreiben

        in      Tmp1,GICR               ;General Int. Control Reg. einlesen
        andi    Tmp1,0b01011111         ;INT1 = INT2 = 0
        ori     Tmp1,0b01000000         ;INT0=1 => INT0 aktivieren
        out	GICR,Tmp1		;GICR setzen

        ;USART initialisieren
        ;Empfaenger einschalten, Interrupt freigeben und Datenformat festlegen
        sbi     UCSRB,RXEN      ;Epfaenger einschalten (TxEnable)
        sbi     UCSRB,TXEN      ;Sender einschalten (TxEnable)
        sbi     UCSRB,RXCIE     ;Interrupt RXCIE freigeben
        ldi     Tmp1,0x86       ;8N1
        out     UCSRC,Tmp1      
        ;Baudrate initialisieren
        clr     Tmp1            ;Teiler fuer Baudrate festlegen
        out     UBRRH,Tmp1      ;Higbyte = 0
        ldi     Tmp1,0x19       ;Teile durch dezimal 25 fuer 38400 Baud (16MHz)
        out     UBRRL,Tmp1      ;Lowbyte = 0x19

        ;Interrupt einschalten und freigeben    
        sei                     ;Interrupts global freigeben (I-Bit im SREG) 

;-------------------------------------------------------------------------------
;       Hauptprogramm
;-------------------------------------------------------------------------------
MAIN:   rcall   I2CINI          ;I2C Bus initialisieren auf 100kHz
   	rcall	LCDINI		;Initialisiert das LCD-Interface für 4 Bit
   	rcall	KEYINI		;Initilisiert die Tastatur
        rcall   W1ms
        rcall   RTCRCV		;RTC 2x lesen
        lds     Tmp1,RTCTAB+0	;Falls Uhr nicht laeuft, dann starten
        rcall	W1s
        rcall	W1ms
        rcall   RTCRCV		
        lds     Tmp2,RTCTAB+0        
        cp	Tmp1,Tmp2
        brne	MAIN2
        ;RTC starten 01.01.12 22:33:00
        ldi     Tmp1,0x00	;Sekunden
        sts     RTCTAB+0,Tmp1
        ldi     Tmp1,0x33	;Minuten
        sts     RTCTAB+1,Tmp1
        ldi     Tmp1,0x22	;Stunden
        sts     RTCTAB+2,Tmp1
        ldi     Tmp1,0x07	;Wochentag (ab Montag)
        sts     RTCTAB+3,Tmp1
        ldi     Tmp1,0x01	;Tag
        sts     RTCTAB+4,Tmp1
        ldi     Tmp1,0x01	;Monat
        sts     RTCTAB+5,Tmp1
        ldi     Tmp1,0x12	;Jahr
        sts     RTCTAB+6,Tmp1
        ldi     Tmp1,0x10	;Control Byte SQW ein (1Hz)
        sts     RTCTAB+7,Tmp1      
        rcall   RTCSND
MAIN2:  rcall   OW1RROM         ;Lese ROMCode (SN) des Temperatursensor
        ldi	r24,0x0D
        rcall	SNDCHR
        ldi	r24,'S'		;Ausgabe serielle Schnittstelle
        rcall	SNDCHR
        ldi	r24,'N'
        rcall	SNDCHR
        ldi	r24,':'
        rcall	SNDCHR
        ldi	r24,' '        
        rcall	SNDCHR
        rcall   ROM232          
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,'D'
        rcall	SNDCHR
        ldi	r24,'S'
        rcall	SNDCHR
        ldi	r24,'1'
        rcall	SNDCHR
        ldi	r24,'8'
        rcall	SNDCHR
        lds	Tmp1,ROMCOD+0
        cpi	Tmp1,0x28
        breq	DS18B
        ldi	r24,'S'
        rcall	SNDCHR
        rjmp	MAIN3
DS18B:  ldi	r24,'B'
        rcall	SNDCHR
MAIN3:  ldi	r24,'2'
        rcall	SNDCHR
        ldi	r24,'0'
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        
        
        rcall	W1s        
	rcall	LCDCLR		;Loescht das Display
        
LOOP:   rcall   OW1MROM         ;Match ROM Code senden
        ldi     r24,0x44        ;Befehl "Convert T" senden
        rcall   OW1SND

	;Warteschleife 1 Sekunde bis Wandlung fertig
	;Waehrend dieser Zeit wird die Tastatur abgefragt!
        
	ldi	Cnt1,10
KL:	rcall	KEYP
	ldi	r16,0x30
	add	r16,KeyCode
	cpi	r16,0x30
	brne	KL1
	ldi	r16,' '
	rjmp	KL4
KL1:	cpi	r16,0x3A
	brne	KL2
	ldi	r16,'*'
	rjmp	KL4
KL2:	cpi	r16,0x3B
	brne	KL3
	ldi	r16,'0'
	rjmp	KL4
KL3:	cpi	r16,0x3C
	brne	KL4
	ldi	r16,'#'
KL4:	ldi     r24,0x8F
        rcall   LCDSCO          ;1 Zeile
	mov	r24,r16
	rcall	LCDOUT
	mov	r24,r16
	rcall	SNDCHR	
        rcall	W100ms
        dec 	Cnt1
        brne	KL

	ldi	r24,0x0D
        rcall	SNDCHR

	rcall   RTCRCV		;RTC lesen
        rcall   SNDTD		;Datum und Uhrzeit auf EIA232 ausgeben         
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
                
        ldi     r24,0x80
        rcall   LCDSCO          ;1 Zeile
        ldi	ZL,LOW(RTCASC)
	ldi	ZH,HIGH(RTCASC)
	ldi	Cnt1,14
DL1:	ld      r24,Z+
        rcall	LCDOUT		;Zeichen senden, Autoinkrement der Adresse
        dec	Cnt1
        brne	DL1	
        
        ldi     r24,0xC0
        rcall   LCDSCO          ;2 Zeile
        ldi	ZL,LOW(TMPASC1+1)
	ldi	ZH,HIGH(TMPASC1+1)
        rcall	LCDSTR
        ldi	ZL,LOW(TEXT7*2)
	ldi	ZH,HIGH(TEXT7*2)
        rcall	LCDSTF
        



	rcall   OW1MROM                 ;Match ROM senden
        clr	r24                     ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   OW1RSCR                 ;Lese Scratchpad uns speichere im SRAM
        clr	r24                     ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall   S1TTOA                  ;Berechne Temp. und wandele in ASCII
        clr	r24                     ;Offset Scratchp in r24 (beginnt mit 0!)
        rcall	TMP1_232		;zur EIA232
        ldi	r24,'G'
        rcall	SNDCHR
        ldi	r24,'r'
        rcall	SNDCHR
        ldi	r24,'a'
        rcall	SNDCHR
        ldi	r24,'d'
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,'C'
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        ldi	r24,' '
        rcall	SNDCHR
        

	rjmp	LOOP            ;Endlosschleife


;ENDE:	rjmp ENDE        
;-------------------------------------------------------------------------------
;       Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
RXINT:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16             ;
	push    r17    
	push    r18    
	push    XH
	push    XL
	
	;"T " gesendet?	
	in      r16,UDR         ;Zeichen einlesen	
	cpi	r16,'T'
	breq    RXINT1
	rjmp	RXINTR
RXINT1: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT1          ;(Polling)
        in      r16,UDR         ;Lese Datenbyte ein
        cpi	r16,' '
	breq    RXINT2
	rjmp	RXINTR
	;Tag
RXINT2: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT2          ;(Polling)
        in      r16,UDR         ;Lese Datenbyte ein
	subi	r16,0x30
	swap	r16
RXINT3: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT3          ;(Polling)
        in      r17,UDR         ;Lese Datenbyte ein
	subi	r17,0x30
	add	r16,r17
        sts     RTCTAB+4,r16
	;Monat
RXINT4: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT4          ;(Polling)
        in      r16,UDR         ;Lese Datenbyte ein
	subi	r16,0x30
	swap	r16
RXINT5: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT5          ;(Polling)
        in      r17,UDR         ;Lese Datenbyte ein
	subi	r17,0x30
	add	r16,r17
        sts     RTCTAB+5,r16
	;Jahr
RXINT6: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT6          ;(Polling)
        in      r16,UDR         ;Lese Datenbyte ein
	subi	r16,0x30
	swap	r16
RXINT7: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT7          ;(Polling)
        in      r17,UDR         ;Lese Datenbyte ein
	subi	r17,0x30
	add	r16,r17
        sts     RTCTAB+6,r16
	;Stunden
RXINT8: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT8          ;(Polling)
        in      r16,UDR         ;Lese Datenbyte ein
	subi	r16,0x30
	swap	r16
RXINT9: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINT9          ;(Polling)
        in      r17,UDR         ;Lese Datenbyte ein
	subi	r17,0x30
	add	r16,r17
        sts     RTCTAB+2,r16
	;Minuten
RXINTA: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINTA          ;(Polling)
        in      r16,UDR         ;Lese Datenbyte ein
	subi	r16,0x30
	swap	r16
RXINTB: sbis    UCSRA,RXC       ;Warte bis RXC (Receive Complete) gesetzt ist.
        rjmp    RXINTB          ;(Polling)
        in      r17,UDR         ;Lese Datenbyte ein
	subi	r17,0x30
	add	r16,r17
        sts     RTCTAB+1,r16
        
        clr	r16
        sts     RTCTAB+0,r16

	rcall   RTCSND

RXINTR: pop	XL
	pop	XH
	pop	r18
	pop	r17
	pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti                    ;zurueck und Interrupts wieder freigeben
        
ISRI0:  push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16        
        push    r17        
        push    ZL        
        push    ZH
        
	sbis	PINC,3		;LED toggeln
	rjmp	ISRI0S
	cbi	PORTC,3
	rjmp    ISRI0R
ISRI0S: sbi	PORTC,3

  	;Adresszeiger initialisieren
        ldi     ZL,LOW(SMTAB*2) ;Adresszeiger mit der Adresse der Dekodiertab.
        ldi     ZH,HIGH(SMTAB*2); * 2 initialisieren (Worte statt Bytes)        
        add	ZL,Cnt2
        adc	ZH,Zero
                
STEP:   lpm     r16,Z          ;Wert aus Tabelle lesen und ausgeben
        in	r17,PIND
        or	r17,r16
        ori	r16,0x0F
        and	r17,r16
        out	PORTD,r17
        inc     Cnt2
        cpi     Cnt2,4          
        brne    ISRI0R
        clr	Cnt2        

ISRI0R: rcall	W10ms
	in	r17,PIND
        andi	r17,0x0F	;Stromlos
        out	PORTD,r17

	pop	ZH
	pop	ZL
	pop	r17
	pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        ;
        pop     r16             ;r16 wiederherstellen
        reti                    ;zurueck und Interrupts wieder freigeben

SNDCHR: sbis	UCSRA,UDRE	;Polling ob frei
	rjmp	SNDCHR
        out     UDR,r24         ;Gib ein Zeichen aus        
        ret

.INCLUDE "SR_1_WIRE.asm"        ;One Wire Bibliothek einbinden
.INCLUDE "SR_TIME_16M.asm"      ;Zeitschleifenbibliothek einbinden
.INCLUDE "SR_I2C.asm"           ;I2C-Bibliothek einbinden
.INCLUDE "SR_RTC_DS1307.asm"    ;RTC-Bibliothek einbinden
.INCLUDE "SR_LCD_2x16_4Bit.asm" ;LCD-Bibliothek einbinden
.INCLUDE "SR_KEYPAD_3x4.asm" ;LCD-Bibliothek einbinden

;-------------------------------------------------------------------------------
;       Tabellen im Programmspeicher (Flash)
;-------------------------------------------------------------------------------
SMTAB:
.DB     0x50,0x60,0xA0,0x90

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT                           ;Ende des Quelltextes

